#include <pthread.h>
#include <windows.h>
#include <stdio.h>
#include <stdlib.h>

DWORD GetCpuCount() 
{
	// Struktura sa osnovim podacima o sistemu.
	SYSTEM_INFO siSysInfo;

	// Dohvati podatke o aktuelnom sistemu. 
	GetSystemInfo(&siSysInfo); 

#ifdef _DEBUG
	// Prikaži sve podatke. 
	printf("Hardware information: \n");  
	printf("  OEM ID: %u\n", siSysInfo.dwOemId);
	printf("  Number of processors: %u\n", siSysInfo.dwNumberOfProcessors); 
	printf("  Page size: %u\n", siSysInfo.dwPageSize); 
	printf("  Processor type: %u\n", siSysInfo.dwProcessorType); 
	printf("  Minimum application address: %lx\n", siSysInfo.lpMinimumApplicationAddress); 
	printf("  Maximum application address: %lx\n", siSysInfo.lpMaximumApplicationAddress); 
	printf("  Active processor mask: %u\n", siSysInfo.dwActiveProcessorMask); 
#endif

	return siSysInfo.dwNumberOfProcessors;
}

void MapPthreadToCpu(pthread_t pthread, unsigned cpuNumber)
{
	DWORD affinityMask;
	BOOL stamResult = 0;
	HANDLE pthreadHandle;
	
	// Dođi do native niti
	pthreadHandle = pthread_getw32threadhandle_np(pthread);

	// Podesi traženi procesor
	affinityMask = 1UL << cpuNumber;
	if (stamResult = SetThreadAffinityMask(pthreadHandle, affinityMask))
	{
		printf("Nit %p postavljena na CPU %u\n", pthread.p, cpuNumber + 1);
	}
	else
	{
		printf("Nema ništa od ovog posla za nit %p, uzrok je %u\n", pthread.p, GetLastError());
	}
}

#define NUM_THREADS	5

/*
You will need to call pthread_win32_process_attach_np() before
you can call any pthread routines when statically linking.
You should call pthread_win32_process_detach_np() before
exiting your application to clean up.
*/
/*
Set C/C++->General->Detect 64 Bit Portability Issus to "false" to aviod warnings
(this will remove /Wp64 from compiler options)
*/

void *PrintHello(void *threadid)
{
	int i, j = rand()*rand()*rand();
	printf("Ušao u nit %d\n", threadid);

	// za niti koje su na parnim procesorima, prvo spavanje, pa onda rad
	if (0 == (int)threadid % 2)
	{
		printf("Počinjem dremku\n");
		Sleep(10000);
		printf("Nit %d se upravo probudila\n",threadid); 
	}

	printf("Počinjem nepotrebni ciklus\n");
	for (i = 0; i < j; i++)
		i++;
	printf("\n%d: Hello World!\n", threadid);
	pthread_exit(NULL);
	return 0;
}

int main(int argc, char *argv[])
{
	pthread_t threads[NUM_THREADS];
	int rc, t, i, j = rand()*rand()*100;

	DWORD cpuCount;
	int posixCpuCount;

	pthread_win32_process_attach_np();

	cpuCount = GetCpuCount();
	posixCpuCount = pthread_num_processors_np();

	for(t=0;t<NUM_THREADS;t++)
	{
		printf("Stvaram POSIX nit %d\n", t);
		// Napravi POSIX nit.
		rc = pthread_create(&threads[t], NULL, PrintHello, (void *)t);
		if (rc)
		{
			printf("GREŠKA: povratna vrednost iz pthread_create(): %d\n", rc);
			exit(-1);
		}

		// Postavi na željeni CPU.
		MapPthreadToCpu(threads[t], t % cpuCount);

		for (i = 0; i < j; i++) i++;

	}

	pthread_exit(NULL);

	pthread_win32_process_detach_np();
}
